%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Task Force on Modern Heuristic Optimization Test Beds
% Working Group on Modern Heuristic Optimization
% Intelligent Systems Subcommittee
% Power System Analysis, Computing, and Economic Committee
%
% 2018 Grid Optimization Competition
% Evaluating the Performance of Modern Heuristic Optimizers on Stochastic Optimization Problems applied to Smart Grids
%
% Test bed A:
% Stochastic OPF in Presence of Renewable Energy and Controllable Loads
%
% Based on 2014 and 2017 competition AND [1], [2]
%
% [1] J. Arvalo, F. Santos, S. Rivera, Uncertainty cost functions for solar photovoltaic generation, wind
% energy generation, and plug-in electric vehicles: Mathematical expected value and verification by monte
% carlo simulation, International Journal of Power and Energy Conversion, in press, 2018.
% http://www.inderscience.com/info/ingeneral/forthcoming.php?jcode=ijpec
% [2] S. Rivera, A. Romero, Dispatch Modeling Incorporating Maneuver Components, Wind Power and
% Electric Vehicles Penetration, using Heuristic Techniques, Application of Modern Heuristic Optimization
% Techniques in Power and Energy Systems, ed: Wiley, in press, 2018.
%
% Questions can be directed to Sergio Rivera srriverar@unal.edu.co
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

                    %% WIND PARAMETERS

                    Vi=5;           
                    Vr=15;          
                    Vo=45;          
                    Wr=150;         
                    a= 15;          
                    b= -75;         
                    sg= 15.9577;    
                    c=sqrt(2)*sg;   
                    k= 2;           
                   
                    %% SOLAR PARAMETERS                    

                    Psr= 100; 
                    Gstd= 775; 
                    Rc=116;  
                    Wmax= 150; 
                    lambda= 6;  
                    beta= 0.25;
                    mu=lambda;  
                    sigma=beta;

                    %% A SET OF MONTECARLO SIMULATIONS
                    
                    N=2000;        
                    
for hi=1:N
                        v2(hi)=wblrnd(c,k); %% ALEATORY WIND SPEED
                        G(hi)=lognrnd(mu,sigma); %% ALEATORY SOLAR IRRADIATION
                        v6(hi)=wblrnd(c,k); %% ALEATORY WIND SPEED

                        if (v2(hi)<Vi)||(v2(hi)>Vo)
                            W2(hi)=0;
                        elseif v2(hi)<Vr && Vi<v2(hi)
                            W2(hi)=a*v2(hi)+b;
                        else
                            W2(hi)=Wr;
                        end

                        if ((G(hi)<Rc)&&(G(hi)>0))
                                W4(hi)=Psr*(G(hi)^2/(Gstd*Rc));
                        elseif G(hi)>Rc
                                W4(hi)=Psr*(G(hi)/Gstd);
                        end

                        if (v6(hi)<Vi)||(v6(hi)>Vo)
                            W6(hi)=0;
                        elseif v6(hi)<Vr && Vi<v6(hi)
                            W6(hi)=a*v6(hi)+b;
                        else
                            W6(hi)=Wr;
                        end
end

   figure(1)
   subplot(2,1,1)
   hist(v2)
   xlabel('wind speed (m/s) for wind generator at bus 2')
   ylabel('frequency')
   subplot(2,1,2)
   hist(W2)
   xlabel('available real power (MW) for wind generator at bus 2')
   ylabel('frequency')
 
   figure(2)
   subplot(2,1,1)
   hist(G)
   xlabel('solar irradiance (W/m^2) for solar generator at bus 6')
   ylabel('frequency')
   subplot(2,1,2)
   hist(W4)
   xlabel('available real power (MW) for solar generator at bus 6')
   ylabel('frequency')
   
   figure(3)
   subplot(2,1,1)
   hist(v6)
   xlabel('wind speed (m/s) for wind generator at bus 9')
   ylabel('frequency')
   subplot(2,1,2)
   hist(W6)
   xlabel('available real power (MW) for wind generator at bus 9')
   ylabel('frequency')

